#include "induct.h"

/*=================================================
       evaluation of Pmn used to get the Q's
***************************************************/

/* This function evaluates the definite 4-tuple integral of Green's function for
   a segment.  */

double P(seg *ef, seg *ep)
{
  double a;
/*
  double x,dx1,dx2,dxf,dxp,dy1,dy2,dyf,dyp;
*/
  double nx[4], ey[4];
  double e,n,p,e2,n2,t,f;

  /* just because it is easier to write and maybe faster... */

  int sign[4][4] =
                 {{  1, -1, -1,  1}, 
                  { -1,  1,  1, -1}, 
		  { -1,  1,  1, -1}, 
		  {  1, -1, -1,  1}};
  int i,j;

/*

dx2 = ef->xur - ep->xur;
dxf = ef->xll - ep->xur;
dxp = ef->xur - ep->xll;
dx1 = ef->xll - ep->xll;

dy2 = ef->yur - ep->yur;
dyf = ef->yll - ep->yur;
dyp = ef->yur - ep->yll;
dy1 = ef->yll - ep->yll;
*/

/* Now to evaluate the 4-definite integral with the above limits 
   This requires 16 evaluations of f, since there are 2^4 evaluations,
   permutations of the 8 limits */

/*
  x = K * ef->area_r * ep->area_r;

  a = x * (   F(dx2,dy2) - F(dxf,dy2) 
            - F(dx2,dyf) + F(dxf,dyf) 

            - F(dxp,dy2) + F(dx1,dy2) 
            + F(dxp,dyf) - F(dx1,dyf) 


            - F(dx2,dyp) + F(dxf,dyp) 
            + F(dx2,dy1) - F(dxf,dy1) 
  
            + F(dxp,dyp) - F(dx1,dyp) 
            - F(dxp,dy1) + F(dx1,dy1));
  return(a);
*/

  nx[0] = ef->xur - ep->xur;
  nx[1] = ef->xur - ep->xll;
  nx[2] = ef->xll - ep->xur;
  nx[3] = ef->xll - ep->xll;

  ey[0] = ef->yur - ep->yur;
  ey[1] = ef->yur - ep->yll;
  ey[2] = ef->yll - ep->yur;
  ey[3] = ef->yll - ep->yll;

  a = 0.0;

  for(i = 0; i < 4; i++)
    for(j = 0; j < 4; j++)
      {
      n = nx[i];
      e = ey[j];
      e2 = e * e;
      n2 = n * n;

      if(fabs(e) <= SMALL)
        if(fabs(n) <= SMALL)
          f = 0.0;
        else
          f = log(n2)*n2*n2;
      else if(fabs(n) <= SMALL)
        f = log(e2)*e2*e2;
      else
      {
        p = e * n;
        t = atan(e/n);
        f = log(e2 + n2) * (e2*e2 - 6*p*p + n2*n2) + 25*p*p;
        if (p > 0)
           f = f - 8*p*(e2*(hPI - t) + n2*t);
         else
           f = f - 8*p*(e2*(-hPI - t) + n2*t);
      }

      a = a + sign[i][j] * f;
     } 
     
  return(K * ef->area_r * ep->area_r * a);
}  /* P evaluation to get Q's */
