
#include "induct.h"

void
subdivide()
{
conductor *tmp, *place;
input_conductor *first, *second;
int i,j,total_seg;

  place = cond_head;
  first = input_cond;

/* since GND is the first conductor, and the first element of GND will
be the reference, we don't want to "count" it here */
  i = -1;

  while (first != NULL) 
  {
    total_seg = 0;
    second = first;

    /* count up the total subsegment that will be there for this conductor */
    while ((second != NULL) && (second->cnum == first->cnum))
    {
      total_seg = second->xnum * second->ynum + total_seg;
      second = second->next;
    }

    /* initialize the new conductor */
    tmp = (conductor *) malloc( sizeof(conductor));
    tmp->cnum = first->cnum;
    tmp->i_start = i;
    tmp->i_end = i + total_seg - 1;
    tmp->lam2 = first->lam2;
    tmp->segarray = (seg **) malloc( total_seg * sizeof(seg *));
    for(j=0;j<total_seg;j++)
       tmp->segarray[j] = (seg *) malloc(sizeof(seg));
    tmp->next = NULL;

    /* put the new conductor in the list */
    if(cond_head != NULL)
      place->next = tmp;
    else
      cond_head = tmp;

    place = tmp;

    make_segs(first,place);

    first = second;
    i = tmp->i_end + 1;
  }
} /* subdivide */

void
make_segs(input_conductor *set, conductor *one)
{
 int i,tmp_i,j,k;
 double min,inc,xl,xr,yl,yr;
 int n;
 
  i = 0;

/* need to go through each conductor in each set.  This subroutine does only each
conductor in one set*/
  while(set != NULL)
  {
     switch(set->xtype)
     {
     /*
        This is a little tricky indexing, but it works.  First, it is important to note
     that the order of the segments in the array (and therefore the Q ... maticies) is
     not important.  So, I can order them in any way I like.  If you were trying to get a
     current distribution PICTURE, this would have to be handled more carefully.  

     For the double ended and middle cases (0 and 3) I divide them up into each side
     being one of cases 1 and 2.  If there is an odd number of segments, I still do the
     same thing, but the results should (hopefully) overlap.  Again, this could be tricky
     if order was maitained, but it isn't.  Then, after x is taken care of throughout the
     whole matrix, (array actually) I do the same for y.
     */

       case 0:   
           tmp_i = i;
           /* get 1/2 the number of segments */
           if(!(001 & set->xnum))
           {
            n = set->xnum/2;
            /* and the corresponding min delta-x */
            min = (set->xur - set->xll) / (2 * R(n,set->xratio));
           }
           else
           {
            n = (set->xnum + 1)/2;
             /* special R(n,r) + R(n-1,r) to get the smallest division for odd n only */
            min = (set->xur - set->xll) / (R(n,set->xratio) + R(n-1,set->xratio));
           }
           
           /* go from left to middle first */
           inc = min;
           xl = set->xll;
           xr = xl + inc;
           /* for each x block... */
           for(j = tmp_i; j < (tmp_i + n); j++)
           {
             /* go through all the (later) y blocks (2-D rectangular array of segments) */
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             /* update inc for update of x*/
             inc = inc*set->xratio;
             /* go to the next x values */
             xl = xr;
             xr = xl + inc;
           }

           if(!(001 & set->xnum))
            tmp_i = tmp_i + n;
           else
           /* if odd total segments, write over the last column (or whatever) 
              so total # segments will be correct */
            tmp_i = tmp_i + n - 1;
           
           /* then go from right to middle */
           inc = min;
           xr = set->xur;
           xl = xr - inc;
           for(j = tmp_i; j < (tmp_i + n); j++)
           {
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             /* update inc for update of x*/
             inc = inc*set->xratio;
             /* go to the next x values */
             xr = xl;
             xl = xr - inc;
           }
           break;

       case 1 :
           /* this is like 1 segment of above, without having 2 subdivisions or
           odd/even problems:  going (small) right to left */
           inc = (set->xur - set->xll) / R(set->xnum,set->xratio);
           xr = set->xur;
           xl = xr - inc;
           for(j = i; j < (i + set->xnum); j++)
           {
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             inc = inc*set->xratio;
             xr = xl;
             xl = xr - inc;
           }
           break;

       case 2:
           /* this is like 1 segment of case 0 above, without having 2 subdivisions or
           odd/even problems:  going (small) left to right */
           inc = (set->xur - set->xll) / R(set->xnum,set->xratio);
           xl = set->xll;
           xr = xl + inc;
           for(j = i; j < (i + set->xnum); j++)
           {
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             inc = inc*set->xratio;
             xl = xr;
             xr = xl + inc;
           }
           break;

       case 3:
           tmp_i = i;
           /* this is like case 0 above, but with small in the middle */
           if(!(001 & set->xnum))
           {
            n = set->xnum/2;
            min = (set->xur - set->xll) / (2 * R(n,set->xratio));
            /* starting point, middle (and then go right first) */
            xl = (set->xur + set->xll)/2;
           }
           else
           {
            n = (set->xnum + 1)/2;
             /* special 2*(R(n,r) - 1/2) to get the smallest division for odd n only */
             /* diffent from in case 0, because small in middle now */
            min = (set->xur - set->xll) / (2*R(n,set->xratio) - 1);
             /* middle (or kind of) to right */
            xl = (set->xur + set->xll - min)/2;
           }
           
           inc = min;
           xr = xl + inc;
           for(j = tmp_i; j < (tmp_i + n); j++)
           {
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             inc = inc*set->xratio;
             xl = xr;
             xr = xl + inc;
           }

           /* update the array index, this time the same for both odd and even */
           tmp_i = tmp_i + n;

           if(!(001 & set->xnum))
            {
            /* find new starting point (middle -> left)*/
            xr = (set->xur + set->xll)/2;
            inc = min;
            }
           else
           {
           /* This time, we can't write over the last column, but "would" have to
           write over the FIRST one, this is wierd, so instead we WON'T write the
           first column instead, start a little left of middle, increase inc one
           step (since it won't be needed again as the original value), and not
           go as far (decrement n) */
            xr = (set->xur + set->xll - min)/2;
            inc = min*set->xratio;
            n = n - 1;
           }
           
           xl = xr - inc;
           for(j = tmp_i; j < (tmp_i + n); j++)
           {
             for(k = j; k <= (set->xnum * (set->ynum - 1) + j); k = k + set->xnum)
             {
               one->segarray[k]->xll = xl;
               one->segarray[k]->xur = xr;
             }
             inc = inc*set->xratio;
             xr = xl;
             xl = xr - inc;
           }
           break;

       default:
           printf("Error, unknown type\n");
           break;
     }

/* now the same switch (or just about) for y */

     switch(set->ytype)
     {
       case 0:   
           tmp_i = i;
           if(!(001 & set->ynum))
           {
            n = set->ynum/2;
            min = (set->yur - set->yll) / (2 * R(n,set->yratio));
           }
           else
           {
            n = (set->ynum + 1)/2;
            min = (set->yur - set->yll) / (R(n,set->yratio) + R(n-1,set->yratio));
           }
           
           inc = min;
           yl = set->yll;
           yr = yl + inc;
           for(k = tmp_i; k <= (set->xnum * (n - 1) + tmp_i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yl = yr;
             yr = yl + inc;
           }

           if(!(001 & set->ynum))
            tmp_i = tmp_i + n * set->xnum;
           else
            tmp_i = tmp_i + (n - 1) * set->xnum;
           
           inc = min;
           yr = set->yur;
           yl = yr - inc;
           for(k = tmp_i; k <= (set->xnum * (n - 1) + tmp_i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yr = yl;
             yl = yr - inc;
           }
           break;

       case 1:
           inc = (set->yur - set->yll) / R(set->ynum,set->yratio);
           yr = set->yur;
           yl = yr - inc;
           for(k = i; k <= (set->xnum * (set->ynum - 1) + i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yr = yl;
             yl = yr - inc;
           }
           break;

       case 2:
           inc = (set->yur - set->yll) / R(set->ynum,set->yratio);
           yl = set->yll;
           yr = yl + inc;
           for(k = i; k <= (set->xnum * (set->ynum - 1) + i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yl = yr;
             yr = yl + inc;
           }
           break;

       case 3:
           tmp_i = i;
           if(!(001 & set->ynum))
           {
            n = set->ynum/2;
            min = (set->yur - set->yll) / (2 * R(n,set->yratio));
            yl = (set->yur + set->yll)/2;
           }
           else
           {
            n = (set->ynum + 1)/2;
            min = (set->yur - set->yll) / (2*R(n,set->yratio) - 1);
            yl = (set->yur + set->yll - min)/2;
           }
           
           inc = min;
           yr = yl + inc;
           for(k = tmp_i; k <= (set->xnum * (n - 1) + tmp_i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yl = yr;
             yr = yl + inc;
           }

           tmp_i = tmp_i + n * set->xnum;

           if(!(001 & set->ynum))
            {
            yr = (set->yur + set->yll)/2;
            inc = min;
            }
           else
           {
            yr = (set->yur + set->yll - min)/2;
            inc = min*set->yratio;
            n = n - 1;
           }
           
           yl = yr - inc;
           for(k = tmp_i; k <= (set->xnum * (n - 1) + tmp_i); k = k + set->xnum)
           {
             for(j = k; j < (k + set->xnum); j++)
             {
               one->segarray[j]->yll = yl;
               one->segarray[j]->yur = yr;
               one->segarray[j]->area_r = 
                  1 / fabs((one->segarray[j]->xur - one->segarray[j]->xll) * inc);
             }
             inc = inc*set->yratio;
             yr = yl;
             yl = yr - inc;
           }
           break;

       default:
           printf("Error, unknown type\n");
           break;
     }

     /* update the index into the array */
     i = i + (set->xnum * set->ynum);

     /* go to the next conductor of the set */

     if((set->next != NULL) && (set->cnum == set->next->cnum))
       set = set->next;
     else
       set = NULL;
  } /* while set != NULL */

} /* make_segs */


double R(int n, double r)
{
  int i;
  double sum;

  sum = 1;
  for(i = 1;i < n;i++)
   sum = r*sum + 1; 
  return(sum);
} /* R(n,r) = SUM(i=0 to n-1) { r^i }  used to get the smallest subdivision */


